/*
 * VOTRE EN-TÊTE ICI!
 */


#ifndef TP3_CHAINETAB_H
#define TP3_CHAINETAB_H


#include <iostream>


/**
 * Classe pour une structure de données de chaîne de tableau.
 */
template <typename TYPE>
class chainetab
{
private:

    struct cellule
    {
        TYPE* TAB;
        cellule* SUIV;  //cellule suivante, nullptr si dernière cellule
        cellule* PREC;  //cellule précédente, nullptr si première cellule
        cellule(TYPE* tab, cellule* prec, cellule* suiv);
    };

    cellule* DEBUT_CHAINE;    //première cellule, ou nullptr si vide
    cellule* FIN_CHAINE;      //dernière cellule, ou nullptr si vide
    size_t TABSIZE;     //taille des tableaux pointés par vos cellules

    size_t POSPREMIER;  //indice du premier élément dans le premier tableau
    size_t POSDERNIER;  //indice du dernier éléments dans le dernier tableau
    size_t SIZE;        //dimension, i.e. le nb d'éléments insérés

public:


    chainetab(size_t tabsize);
    ~chainetab();
    chainetab(const chainetab& source);

    chainetab& operator=(const chainetab& source);


    TYPE& operator[](size_t i);

    size_t size() const;

    void push_front(const TYPE& val);
    void push_back(const TYPE& val);
    void pop_front();
    void pop_back();


    //pour être une vraie liste, il faudrait pouvoir insérer/supprimer
    //n'importe où.  Ce n'est pas demandé.

    void clear();


    void afficher_contenu();    //Affiche le contenu avec une ligne par sous-tableau.  Un X veut dire "case non-utilisée"

};





/*------------------------------------------------------------------------------
IMPLÉMENTATION
------------------------------------------------------------------------------*/



/**
 * Constructeur de cellule.
 */
template <typename TYPE>
chainetab<TYPE>::cellule::cellule(TYPE* tab, cellule* prec, cellule* suiv)
{
    TAB = tab;
    PREC = prec;
    SUIV = suiv;
}




/**
 * Constructeur qui initialise avec une liste vide.
 * tabsize est la taille des sous-tableaux
 */
template <typename TYPE>
chainetab<TYPE>::chainetab(size_t tabsize)
{
    //implémentez-moi
}




/**
 * Destructeur qui désalloue toute la mémoire et nettoie
 */
template <typename TYPE>
chainetab<TYPE>::~chainetab()
{
    clear();
}





/**
 * Constructeur par copie: met la liste à vide et délègue tout à l'opérateur =
 */
template <typename TYPE>
chainetab<TYPE>::chainetab(const chainetab& source)
{
    (*this) = source;
}


/**
 * Opérateur =
 * Nettoie l'objet courant et copie tout de la source
 */
template <typename TYPE>
chainetab<TYPE>& chainetab<TYPE>::operator=(const chainetab& source)
{
    //par clémence, je vous laisse l'implémentation
    clear();
    DEBUT_CHAINE = nullptr; //normalement votre clear devrait tout remettre à nullptr, mais au cas où :)
    FIN_CHAINE = nullptr;

    this->TABSIZE = source.TABSIZE;
    this->POSPREMIER = source.POSPREMIER;
    this->POSDERNIER = source.POSDERNIER;
    this->SIZE = SIZE;

    cellule* ptr = source.DEBUT_CHAINE;

    //copier chaque cellule + tableau de la source, tout en mettant à jour les pointeurs de la chaîne
    while (ptr != nullptr)
    {
        TYPE* montab = new TYPE[TABSIZE];
        for (int i = 0; i < TABSIZE; ++i)
        {
            montab[i] = ptr->TAB[i];
        }
        cellule* nouvcell = new cellule(montab, FIN_CHAINE, nullptr);
        if (FIN_CHAINE != nullptr)
            FIN_CHAINE->SUIV = nouvcell;
        FIN_CHAINE = nouvcell;
        if (DEBUT_CHAINE == nullptr)
            DEBUT_CHAINE = nouvcell;

        ptr = ptr->SUIV;
    }

    return *this;
}



/**
 * Retourne le i-ème élément.
 */
template <typename TYPE>
TYPE& chainetab<TYPE>::operator[](size_t i)
{
    //implémentez-moi
}



/**
 * Retourne le nombre d'éléments dans votre liste.
 */
template <typename TYPE>
size_t chainetab<TYPE>::size() const
{
    return SIZE;
}



/**
 * Ajouter en début de liste.  Insérer une nouvelle cellule au début si nécessaire.
 */
template <typename TYPE>
void chainetab<TYPE>::push_front(const TYPE& val)
{

    //implémentez moi

}

/**
 * Ajouter en fin de liste.  Ajouter une nouvelle cellule à la fin si nécessaire.
 */
template <typename TYPE>
void chainetab<TYPE>::push_back(const TYPE& val)
{
    //implémentez-moi
}



/**
 * Supprime tous les éléments et remet à l'état d'une liste vide.
 */
template <typename TYPE>
void chainetab<TYPE>::clear()
{
    //implémentez-moi
}






/**
 * Enlève l'élément en tête de liste.  Ne vérifie rien.
 */
template <typename TYPE>
void chainetab<TYPE>::pop_front()
{
    //implémentez-moi
}


/**
 * Enlève l'élément en fin de liste.  Ne vérifie rien.
 */
template <typename TYPE>
void chainetab<TYPE>::pop_back()
{
    //implémentez-moi
}


/**
 * Affiche le contenu avec une ligne par sous-tableau.  Un X veut dire "case non-utilisée"
 */
template <typename TYPE>
void chainetab<TYPE>::afficher_contenu()
{
    std::cout<<"--------------------------------------------------"<<std::endl;
    std::cout<<"SIZE="<<SIZE<<" POSPREMIER="<<POSPREMIER<<" POSDERNIER="<<POSDERNIER<<std::endl;


    cellule* ptr = DEBUT_CHAINE;
    while (ptr != nullptr)
    {
        std::cout<<"[  ";
        /*
         * Ici, on itère sur le sous-tableau pointé par ptr.  On met un X aux position non-utilisées,
         * qui peuvent survenir dans le premier ou dernier sous-tableau.
         */
        size_t depart = 0;
        size_t fin = TABSIZE - 1;
        if (ptr->PREC == nullptr)   //si on est sur la 1re cellule
            depart = POSPREMIER;
        if (ptr->SUIV == nullptr)
            fin = POSDERNIER;       //si on est sur la dernière cellule

        for (size_t i = 0; i < TABSIZE; ++i)
        {
            if (i < depart || i > fin)
                std::cout<<"X";
            else
                std::cout<<ptr->TAB[i];
            std::cout<<"  ";
        }

        std::cout<<"]";
        std::cout<<std::endl;

        ptr = ptr->SUIV;
    }

    std::cout<<"Contenu avec []: ";
    for (size_t i = 0; i < SIZE; i++)
    {
        std::cout<<(*this)[i]<<"  ";
    }
    std::cout<<std::endl;


}



#endif //TP3_CHAINETAB_H
